;;; @file c2nload.s
;;; Commodore C2N tape loader for the C2N232
;;; @author Marko Mkel (msmakela@nic.funet.fi)

;;; Copyright  2002 Marko Mkel
;;;
;;;     This program is free software; you can redistribute it and/or modify
;;;     it under the terms of the GNU General Public License as published by
;;;     the Free Software Foundation; either version 2 of the License, or
;;;     (at your option) any later version.
;;;
;;;     This program is distributed in the hope that it will be useful,
;;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;     GNU General Public License for more details.
;;;
;;;     You should have received a copy of the GNU General Public License
;;;     along with this program; if not, write to the Free Software
;;;     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "host.s"

;; read a bit to the Carry flag
#if c2nrdval = $80
#define c2nrdbit lda c2nread:c2nclrd:cmp #c2nrdval
#else
#define c2nrdbit lda #c2nrdval:and c2nread:c2nclrd:cmp #c2nrdval
#endif

	.text
	*=casbuf
	.byte cashdr
	.word prg
	.word prgend
	.dsb 16, $20

cnt	.word 0		; number of bytes to transfer (negative)
	;; store the received data
store	sta $ffff
adr = store + 1
	inc adr
	bne nowrap
	inc adr+1
nowrap	inc cnt		; more bytes to transfer?
	bne recv
	inc cnt+1
	bne recv
	;; end of transfer => set the end address
	ldx adr
	lda adr+1
	stx bend
	sta bend+1
#if cashdr = 3
	ldy #0
#ifdef clrprog
	clrprog
#endif
	pla
	tax
	pla
#endif
	cli
	jmp (ibsout)

	;; transfer loop
recv	stx c2nwrite	; drop WRITE
	c2nrdlo
	c2nclrd
	sty c2nwrite	; raise WRITE
	lda #$7f
	sta stemp	; initialise the counter
	;; loop => receive bit, drop WRITE, receive bit, raise WRITE
recv01	c2nrdbit
	stx c2nwrite	; drop WRITE
	ror stemp
	c2nrdbit
	sty c2nwrite	; raise WRITE
	ror stemp
	bcs recv01	; got all bits?
	lda stemp
	invert
	bcc store	; branch always

	;; entry point => restore the jump vector, display start address
entry	sei
#if cashdr = 3
	pha
	txa
	pha
#endif
	;; initialise the keyboard buffer
	ldx klen
	stx keylen
kcopy	lda keys-1,x
	dex
	sta keybuf,x
	bne kcopy
	restor		; restore the IBSOUT vector, let C=0
	lda c2nwrite
#if cashdr = 3
	c2nwr1
	sta c2nwrite	; raise WRITE
#endif
	tay
	c2nwr0
	tax
	c2nrdhi
	bcc recv	; jump to the transfer loop (carry is clear)

	.dsb casbuf+182-*, 0
klen	.byte kend-keys
keys	.byte "RUN", $0d	; contents for the keyboard buffer
kend	.dsb casbuf+192-*, 0

	*=ibsout
prg	.word entry
prgend
