/**
 * @file models.h
 * Supported models of 8-bit Commodore computers
 * @author Marko Mkel <msmakela@nic.funet.fi>
 */

/* Copyright  2002 Marko Mkel.

   This file is part of C2NLOAD, a program for loading files to
   Commodore 8-bit computers via the C2N232 device.

   C2NLOAD is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   C2NLOAD is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   The GNU General Public License is often shipped with GNU software, and
   is generally kept in a file called COPYING or LICENSE.  If you do not
   have a copy of the license, write to the Free Software Foundation,
   59 Temple Place, Suite 330, Boston, MA 02111 USA. */

/** The loading algorithm */
enum algo {
  aBASIC,	/**< pre-8032 PET uses a BASIC loader that must be RUN */
  aNORMAL,	/**< PET and VIC-20 do not blank the screen */
  aBLANK,	/**< Commodore 64 and 128 blank the screen */
  a264		/**< The 264 series uses a slightly different protocol */
};

/** Model-specific parameters */
struct model
{
  /** Name of the computer model */
  const char* name;
  /** The loading algorithm */
  enum algo algo;
  /** The 194-byte loader code */
  const unsigned char* loader;
};

/** Null-terminated list of supported models */
extern const struct model models[];
